@@ -15,11 +15,14 @@ from flask import request as flash_request
 
 from shaystack import \
     about, ops, formats, read, nav, watch_sub, \
-    watch_unsub, watch_poll, point_write, his_read, his_write, invoke_action
+    watch_unsub, watch_poll, point_write, his_read, his_write, invoke_action, HaystackInterface
 from shaystack.ops import HaystackHttpRequest, HaystackHttpResponse
 
 
-def create_haystack_bp() -> Blueprint:
+def create_haystack_bp(provider: HaystackInterface) -> Blueprint:
+
+    # test if there is already a created provider and of the caching is enabled
+
     prefix = os.environ.get('URL_PREFIX') if os.environ.get('URL_PREFIX') else ''
     haystack_blueprint = Blueprint('haystack', __name__,
                                    static_folder=safe_join(os.path.dirname(__file__), 'haystackui'),
@@ -32,7 +35,7 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(about(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(about(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/ops', methods=['POST', 'GET'])
     def flask_ops() -> Response:
@@ -41,7 +44,7 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(ops(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(ops(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/formats', methods=['POST', 'GET'])
     def flask_formats() -> Response:
@@ -50,7 +53,8 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(formats(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(formats(envs, _as_request(flash_request),
+                                    envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/read', methods=['POST', 'GET'])
     def flask_read() -> Response:
@@ -59,7 +63,7 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(read(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(read(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/nav', methods=['POST', 'GET'])
     def flask_nav() -> Response:
@@ -68,7 +72,7 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(nav(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(nav(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/watchSub', methods=['POST', 'GET'])
     def flask_watch_sub() -> Response:
@@ -77,7 +81,8 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(watch_sub(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(watch_sub(envs, _as_request(flash_request),
+                                      envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/watchUnsub', methods=['POST', 'GET'])
     def flask_watch_unsub() -> Response:
@@ -87,7 +92,7 @@ def create_haystack_bp() -> Blueprint:
         """
         envs = cast(Dict[str, str], os.environ)
         return _as_response(watch_unsub(envs, _as_request(flash_request),
-                                        envs.get("FLASK_ENV", "prod")))
+                                        envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/watchPoll', methods=['POST', 'GET'])
     def flask_watch_poll() -> Response:
@@ -96,7 +101,8 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(watch_poll(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(watch_poll(envs, _as_request(flash_request),
+                                       envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/pointWrite', methods=['POST', 'GET'])
     def flask_point_write() -> Response:
@@ -106,7 +112,7 @@ def create_haystack_bp() -> Blueprint:
         """
         envs = cast(Dict[str, str], os.environ)
         return _as_response(point_write(envs, _as_request(flash_request),
-                                        envs.get("FLASK_ENV", "prod")))
+                                        envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/hisRead', methods=['POST', 'GET'])
     def flask_his_read() -> Response:
@@ -115,7 +121,8 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(his_read(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(his_read(envs, _as_request(flash_request),
+                                     envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/hisWrite', methods=['POST', 'GET'])
     def flask_his_write() -> Response:
@@ -124,7 +131,8 @@ def create_haystack_bp() -> Blueprint:
             Flask HTTP response
         """
         envs = cast(Dict[str, str], os.environ)
-        return _as_response(his_write(envs, _as_request(flash_request), envs.get("FLASK_ENV", "prod")))
+        return _as_response(his_write(envs, _as_request(flash_request),
+                                      envs.get("FLASK_ENV", "prod"), provider))
 
     @haystack_blueprint.route('/invokeAction', methods=['POST', 'GET'])
     def flask_invoke_action() -> Response:
@@ -134,14 +142,16 @@ def create_haystack_bp() -> Blueprint:
         """
         envs = cast(Dict[str, str], os.environ)
         return _as_response(invoke_action(envs, _as_request(flash_request),
-                                          envs.get("FLASK_ENV", "prod")))
+                                          envs.get("FLASK_ENV", "prod"), provider))
+
     @haystack_blueprint.route('/', methods=['GET'], defaults={'filename': 'index.html'})
     @haystack_blueprint.route('/<path:filename>', methods=['GET'])
     def flash_web_ui(filename) -> Response:
-        if(os.environ.get('HAYSTACK_INTERFACE') and os.environ.get('HAYSTACK_INTERFACE')!='' and filename == 'plugins/customPlugin.js'):
+        if(os.environ.get('HAYSTACK_INTERFACE') and
+                os.environ.get('HAYSTACK_INTERFACE') != ''
+                and filename == 'plugins/customPlugin.js'):
             return send_from_directory(os.getcwd(), os.environ['HAYSTACK_INTERFACE'])
-        else:
-            return send_from_directory(haystack_blueprint.static_folder, filename)
+        return send_from_directory(haystack_blueprint.static_folder, filename)
     return haystack_blueprint
 
 
