@@ -18,7 +18,7 @@ from dataclasses import dataclass, field
 from datetime import datetime, date
 from decimal import Decimal
 from typing import Optional, Any, List, cast
-from typing import Tuple, Dict
+from typing import Tuple, Dict, Union
 
 from accept_types import get_best_match
 from pyparsing import ParseException
@@ -31,7 +31,7 @@ from .grid import Grid, VER_3_0
 from .grid_filter import parse_hs_datetime_format
 from .parser import MODE_ZINC, MODE_HAYSON, MODE_CSV, MODE_JSON, parse_scalar, parse, mode_to_suffix
 from .providers.haystack_interface import (
-    HttpError, get_singleton_provider, parse_date_range,
+    HttpError, parse_date_range, HaystackInterface,
 )
 
 _DEFAULT_VERSION = VER_3_0
@@ -340,10 +340,13 @@ def _manage_exception(
     )
 
 
-def about(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> HaystackHttpResponse:
+def about(envs: Dict[str, str], request: HaystackHttpRequest, stage: str,
+          provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack about.
     Args:
+        envs: The environments variables
+        provider: HaystackInterface provider to be used
         envs: The environments variables
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
@@ -355,7 +358,6 @@ def about(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hay
     log.debug("HAYSTACK_PROVIDER=%s", envs.get("HAYSTACK_PROVIDER", None))
     log.debug("HAYSTACK_DB=%s", envs.get("HAYSTACK_DB", None))
     try:
-        provider = get_singleton_provider(envs)
         if headers["Host"].startswith("localhost:"):
             home = "http://" + headers["Host"] + "/"
         else:
@@ -368,11 +370,13 @@ def about(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hay
     return response
 
 
-def ops(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> HaystackHttpResponse:
+def ops(envs: Dict[str, str], request: HaystackHttpRequest, stage: str,
+        provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack `ops`.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -381,7 +385,6 @@ def ops(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hayst
     """
     headers = request.headers
     try:
-        provider = get_singleton_provider(envs)
         grid_response = provider.ops()
         assert grid_response is not None
         response = _format_response(headers, grid_response, 200, "OK")
@@ -391,11 +394,12 @@ def ops(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hayst
 
 
 def formats(envs: Dict[str, str], request: HaystackHttpRequest,
-            stage: str) -> HaystackHttpResponse:
+            stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'formats'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -404,7 +408,6 @@ def formats(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers = request.headers
     try:
-        provider = get_singleton_provider(envs)
         grid_response = provider.formats()
         if grid_response is None:
             grid_response = Grid(
@@ -445,11 +448,29 @@ def formats(envs: Dict[str, str], request: HaystackHttpRequest,
     return response
 
 
-def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> HaystackHttpResponse:
+def convert_version(version: Union[datetime, date]) -> datetime:
+    """
+    Converts the given version to str and uses 23:59:59 for the time when the version
+    is based only on the date in order to get the most recent version at that date.
+
+    :param version: either date or datetime version of ontology file in S3
+    """
+    if not isinstance(version, datetime):
+        version = datetime.combine(version, datetime.min.time()).replace(
+            hour=23, minute=59, second=59)
+    elif version.hour == 0 & version.minute == 0 & version.second == 0:
+        version = version.replace(hour=23, minute=59, second=59)
+
+    return version
+
+
+def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str,
+         provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack `read`
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -458,7 +479,6 @@ def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hays
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         read_ids: Optional[List[Ref]] = None
         select = read_filter = date_version = None
@@ -479,7 +499,6 @@ def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hays
             date_version = (
                 grid_request[0].get("version", None) if grid_request else None
             )
-
         # Priority of query string
         if args:
             if "id" in args:
@@ -493,7 +512,7 @@ def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hays
                 select = args["select"]
             if "version" in args:
                 date_version = parse_hs_datetime_format(args["version"], default_tz)
-
+                date_version = convert_version(date_version)
         if read_filter is None:
             read_filter = ""
         if read_ids is None and read_filter is None:
@@ -514,11 +533,13 @@ def read(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hays
     return response
 
 
-def nav(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> HaystackHttpResponse:
+def nav(envs: Dict[str, str], request: HaystackHttpRequest, stage: str,
+        provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'nav'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -527,7 +548,6 @@ def nav(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hayst
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         nav_id = None
         if grid_request and "navId" in grid_request.column:
@@ -543,11 +563,12 @@ def nav(envs: Dict[str, str], request: HaystackHttpRequest, stage: str) -> Hayst
 
 
 def watch_sub(envs: Dict[str, str], request: HaystackHttpRequest,
-              stage: str) -> HaystackHttpResponse:
+              stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'watchSub'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -556,7 +577,6 @@ def watch_sub(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         watch_dis = watch_id = lease = None
         ids = []
@@ -589,11 +609,12 @@ def watch_sub(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def watch_unsub(envs: Dict[str, str], request: HaystackHttpRequest,
-                stage: str) -> HaystackHttpResponse:
+                stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'watchUnsub'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -602,7 +623,6 @@ def watch_unsub(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         close = False
         watch_id = False
@@ -633,11 +653,12 @@ def watch_unsub(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def watch_poll(envs: Dict[str, str], request: HaystackHttpRequest,
-               stage: str) -> HaystackHttpResponse:
+               stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'watchPoll'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -646,7 +667,6 @@ def watch_poll(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         watch_id = None
         refresh = False
@@ -670,11 +690,12 @@ def watch_poll(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def point_write(envs: Dict[str, str], request: HaystackHttpRequest,
-                stage: str) -> HaystackHttpResponse:
+                stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'pointWrite'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -683,7 +704,6 @@ def point_write(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         date_version = None
         level = 17
@@ -736,11 +756,12 @@ def point_write(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def his_read(envs: Dict[str, str], request: HaystackHttpRequest,
-             stage: str) -> HaystackHttpResponse:
+             stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'hisRead'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -749,7 +770,6 @@ def his_read(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         entity_id = date_version = None
         date_range = None
@@ -771,6 +791,7 @@ def his_read(envs: Dict[str, str], request: HaystackHttpRequest,
                 date_range = args["range"]
             if "version" in args:
                 date_version = parse_hs_datetime_format(args["version"], default_tz)
+                date_version = convert_version(date_version)
 
         grid_date_range = parse_date_range(date_range, provider.get_tz())
         log.debug(
@@ -791,11 +812,12 @@ def his_read(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def his_write(envs: Dict[str, str], request: HaystackHttpRequest,
-              stage: str) -> HaystackHttpResponse:
+              stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'hisWrite'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -804,7 +826,6 @@ def his_write(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         entity_id = grid_request.metadata.get("id")
         date_version = grid_request.metadata.get("version")
@@ -834,11 +855,12 @@ def his_write(envs: Dict[str, str], request: HaystackHttpRequest,
 
 
 def invoke_action(envs: Dict[str, str], request: HaystackHttpRequest,
-                  stage: str) -> HaystackHttpResponse:
+                  stage: str, provider: HaystackInterface) -> HaystackHttpResponse:
     """
     Implement Haystack 'invokeAction'.
     Args:
         envs: The environments variables
+        provider: HaystackInterface provider to be used
         request: The HTTP Request
         stage: The current stage (`prod`, `dev`, etc.)
 
@@ -847,7 +869,6 @@ def invoke_action(envs: Dict[str, str], request: HaystackHttpRequest,
     """
     headers, args = (request.headers, request.args)
     try:
-        provider = get_singleton_provider(envs)
         grid_request = _parse_body(request)
         entity_id = grid_request.metadata.get("id")
         action = grid_request.metadata.get("action")
